#!/usr/local/bin/ruby
# Ruby Unit Tests

# Copyright 2001 by Jim Weirich (jweirich@one.net).
# All rights reserved.

# Permission is granted for use, copying, modification, distribution,
# and distribution of modified versions of this work as long as the
# above copyright notice is included.

require 'runit/testcase'
require 'runit/cui/testrunner'

$last_suite = RUNIT::TestSuite.new

######################################################################
class TestMethodFinder < RUNIT::TestCase

  def test_create
    finder = Gem::MethodFinder.new
    assert_not_nil finder
  end
end

$last_suite.add_test (TestMethodFinder.suite)


######################################################################

if __FILE__ == $0 then
  RUNIT::CUI::TestRunner.quiet_mode = true
  RUNIT::CUI::TestRunner.run ($last_suite)
end
