#!/usr/bin/env ruby
# -*- ruby -*-

# Copyright 2000, 2001 by Jim Weirich (jweirich@one.net).
# All rights reserved.

# Permission is granted for use, copying, modification, distribution,
# and distribution of modified versions of this work as long as the
# above copyright notice is included.

######################################################################
# Classes and modules used for testing gemfinder.
#

module Mone
  def m1
  end
end

module Mtwo
  def m2
  end
end

class Parent
  include Mone

  def pub
  end

  protected

  def prot
  end
  
  private

  def priv
  end

  def Parent.pc
  end
end

class Child < Parent
  include Mtwo

  def cpub
  end

  private

  def cpriv
  end

  def Child.cc
  end
end

class Sibling < Parent
end

