#!/usr/bin/env ruby
# -*- ruby -*-

# Copyright 2001 by Jim Weirich (jweirich@one.net).
# All rights reserved.

# Permission is granted for use, copying, modification, distribution,
# and distribution of modified versions of this work as long as the
# above copyright notice is included.

# GemFinder Version Number
REV = "1.9.6"
