#!/usr/bin/env ruby
# -*- ruby -*-

# Copyright 2001 by Jim Weirich (jweirich@one.net).
# All rights reserved.

# Permission is granted for use, copying, modification, distribution,
# and distribution of modified versions of this work as long as the
# above copyright notice is included.


######################################################################
# Extensions to Kernel
#
module Kernel
  def when_missing
    self
  end
end


######################################################################
# Extensions to NilClass
#
class NilClass
  def when_missing
    yield
  end
end
